$!
$! EDIT_LAVC.COM  -- Edit SYS$EXAMPLES:LAVC$FAILURE_ANALYSIS.MAR and create a
$!                   customized version for this particular cluster, and put
$!                   that customized version into the user's default directory.
$!
$!	After the LAVC$FAILURE_ANALYSIS.MAR file is created, re-run this
$!	procedure with a P1 parameter of "BUILD" to assemble/compile and link
$!	the resulting program.  You'll need to compile or assemble & link once
$!	for each of the different CPU architectures in the cluster.  The Macro
$!	code is identical for all architectures, so that only need be generated
$!	once.
$!
$!	Note that this procedure creates code for LAVC$FAILURE_ANALYSIS.MAR
$!	based on the SCS LAN configuration that it sees when it is run.  If
$!	some piece of hardware is broken at the time this is run, the 
$!	procedure will consider that situation normal.  Given that folks will
$!	first tend to run this procedure when they start having LAN problems,
$!	it's a good idea to at least look at the ASCII-Art drawing the
$!	procedure places as comments in the generated code to make sure that
$!	all the adapters, links, segments, and connections that should be
$!	in place are shown in the drawing.
$!
$!	It is necessary to re-run this program and re-create the code any time
$!	any of the following occurs:
$!	o  A LAN adapter which runs SCS is added to or removed from a system
$!	o  A LAN adapter is repaired or replaced and the MAC address changes
$!	o  A node is added to or removed from the cluster
$!	o  SCS comminications is enabled or disabled on a LAN adapter, either
$!	   using SYS$EXAMPLES:LAVC$STOP_BUS or LAVC$START_BUS or SCACP
$!	o  The network connecting the systems is reconfigured in any way that
$!	   affects which adapters are connected to which LANs, Virtual LANs or
$!	   extended LANs.
$!	To minimize disruptions during these sort of changes, you may want
$!	to run the LAVC$FAILURE_OFF.MAR program (likely found at the same place
$!	you obtained this program) to turn off LAVC$FAILURE_ANALYSIS before
$!	making the change, and then after the change has been implemented,
$!	re-run this procedure, verify the output, re-build and then run the
$!	resulting new .EXE on all nodes to resume monitoring.
$!
$!	This procedure will generate descriptions for nodes, adapters, and
$!	VLANs.  These descriptions will appear in OPCOM messages when failures
$!	and repairs occur.  If you don't like the default descriptions, you
$!	can use logical names to override them during a run of this procedure
$!	(or of course you can edit the resulting .MAR file to your heart's
$!	content afterward, because the main purpose of this procedure was to
$!	create a working example program which could be demonstrated and then
$!	edited by the system manager to provide more detail (and thus better
$!	diagnostic info) on the network configuration that we're able to
$!	"see" looking out from within the systems).  The logical names which
$!	can be defined to override the default descriptions are:
$!	  EDIT_LAVC_DESC_CLUSTER can be defined to specify the cluster name
$!	  EDIT_LAVC_DESC_NODE_xxxx (where "xxxx" is the nodename)
$!	  EDIT_LAVC_DESC_NODE_xxxx_ADAPTER_yyy (where "yyy" is the device name
$!		without	a number (e.g. EIA or FWA), and "xxxx" is the nodename)
$!	  EDIT_LAVC_DESC_VLAN_n (where "n" is the number of the VLAN, starting
$!		with 1 and ascending from there
$!
$!	Procedure requires privileges OPER (for SYSMAN) and CMKRNL (for SDA).
$!
$!	Much of this procedure's data is gathered by parsing SDA output, so
$!	if the format of SDA output changes, this may break.  My apologies
$!	in advance.  Check the website for a later version; failing that,
$!	send me e-mail.
$! Author: Keith Parris  parris@encompasserve.org or keithparris@yahoo.com
$!                       or Keith.Parris@HP.com
$! http://www.geocities.com/keithparris/ or http://encompasserve.org/~parris/
$!
$!	Version is:						V1.1a 3/2003
$	edit_lavc_version = "V1.1a (March 4, 2003)"
$!
$!	Modification history:
$!
$!	V1.0 December, 2002	Initial version, based on logic from
$!				SHOW_ALL_CHANNELS.COM for gathering path
$!				data, SHOW_PATHS.COM for connection logic,
$!				SHOW_SEGMENTS.COM for VLAN-detection logic
$!				and ASCII Art for the network diagram,
$!				SHOW_CONNECTIONS.COM to show node/adapter and
$!				adapter/VLAN connections in the format used
$!				by LAVC$FAILURE_ANALYSIS.MAR, [DO_]SHOWLAN.COM
$!				to gather MAC address data for LAN adapters,
$!				and PARSELAN_LAVC.COM to generate node and
$!				adapter info in LAVC$FAILURE_ANALYSIS format.
$!	V1.1 December, 2002	Better error handling and reporting.  Also
$!				provide version, etc. info in the generated
$!				output file.
$!	V1.1a March, 2003	Better privilege checking.
$!
$	if_debug = "!"	!Use "!" for normal operations, and "" for debugging
$!
$! See if we're running in a cluster.  If not, there's no point to this.
$	if f$getsyi("VAXCLUSTER") .le. 0
$	then
$	    write sys$output "This system is not part of an OpenVMS Cluster."
$	    exit
$	endif
$!
$! Make temporary filenames unique by PID
$	pid = f$getjpi("","PID")
$	temp_file := edit_lavc_temp_'pid'.temp_file
$	delete := delete
$!
$! If this procedure is being called recursively, or to build the program, go
$! do what is needed -->
$	if p1 .eqs. "SHOW_ALL_CHANNELS" -
      .or. p1 .eqs. "SHOWLAN" -
      .or. p1 .eqs. "BUILD" -
		then goto 'p1'
$!
$! This is the main stream of execution
$!
$! See if this cluster uses the LAN for SCS communications or not
$	if (.not. f$getdvi("PEA0","EXISTS")) .or. -
	   (.not. f$getsyi("NISCS_LOAD_PEA0"))
$	then
$	    write sys$output "LAN SCS communications is disabled on this node."
$!	    write sys$output "NISCS_LOAD_PEA0 parameter value is ",-
$!_!			f$getsyi("NISCS_LOAD_PEA0"),"."
$! It's conceivable that there may be a cluster where other interconnects
$! than the LAN are used to talk to a subset of nodes, and yet it is desired
$! that LAVC$FAILURE_ANALYSIS be in place.  In that case, simply run this
$! procedure on one of the nodes which DOES use the LAN for SCS communications.
$	    exit
$	endif
$!
$! Make sure the template program (provided with VMS in the SYS$EXAMPLES:
$! directory) is in place so we can use it as the source of the Macro-32 code
$! we are creating a customized version of (here in our default directory; we
$! don't want to change the original version that is in SYS$EXAMPLES:).
$	if f$search("SYS$EXAMPLES:LAVC$FAILURE_ANALYSIS.MAR") .eqs. ""
$	then
$	    write sys$output -
		"Can't find SYS$EXAMPLES:LAVC$FAILURE_ANALYSIS.MAR."
$	    exit
$	endif
$!
$! Check for the privileges we'll need to drive this process -- most
$! importantly, OPER for SYSMAN here, and CMKRNL to use SDA on each node
$! to gather data
$!
$	auth_privs = "," + f$getjpi("","AUTHPRIV") + ","
$	cur_privs  = "," + f$getjpi("","CURPRIV")  + ","
$! Check for OPER privilege and turn it on if need be
$	OPER_TURNED_ON = "F"	!Assume process already had OPER privilege turned on
$	if f$locate(",OPER,",cur_privs) .eq. f$length(cur_privs)
$	then	!OPER privilege is not presently turned on
$		if f$locate(",OPER,",auth_privs) .ne. f$length(auth_privs) .or. -
		   f$locate(",SETPRV,",auth_privs) .ne. f$length(auth_privs)
$		then
$			set process/privilege=(OPER)
$			OPER_TURNED_ON = "T"
$		else
$			write sys$output -
		    "This procedure requires OPER privilege to run SYSMAN."
$			exit
$		endif
$	endif
$! Check for CMKRNL privilege rights
$	if f$locate(",CMKRNL,",cur_privs) .eq. f$length(cur_privs)
$	then	!CMKRNL privilege is not presently turned on
$		if f$locate(",CMKRNL,",auth_privs) .ne. f$length(auth_privs) .or. -
		   f$locate(",SETPRV,",auth_privs) .ne. f$length(auth_privs)
$		then	!CMKRNL is authorized
$		else
$			write sys$output -
		    "This procedure requires CMKRNL privilege to run SDA."
$			goto cleanup
$		endif
$	endif
$!
$	proc = f$environment("procedure")
$	proc_dir = f$parse(proc,,,"DEVICE") + f$parse(proc,,,"DIRECTORY")
$!
$! Track if data-collection errors occur, and warn the user
$!!	dc_errors = "FALSE"	!Assume innocent until proven guilty
$!! The symbol 'dc_errors' is not used anymore...
$!
$! Gather LAN adapter MAC address info using a recursive call to this
$! procedure (with the SHOWLAN parameter) on each cluster node
$!
$	close/nolog com
$	open/write com 'temp_file'_com
$	write com "$ mcr sysman set environment/cluster"
$	write com "set profile/priv=(cmkrnl)"
$	write com "SET TIMEOUT 0:0:30"  !Give up after 30 seconds
$	write com "do @",proc," SHOWLAN"	!Call us recursively
$	write com "exit"
$	write com "$ exit"
$	close com
$	@'temp_file'_com/out='temp_file'_mac
$	delete 'temp_file'_com;*
$ 'if_debug' type/page/header 'temp_file'_mac	!!!
$! See if there were any error messages in the output
$	define/user sys$output _NLA0:
$	define/user sys$error _NLA0:
$	search/out=_NLA0: 'temp_file'_mac -
		"This procedure requires",-
		error,"-f-","-e-","-w-"
$	if $severity .eq. 1	!1 means string found; 3 means none found
$	then	!Got some errors during data collection
$		write sys$output -
	"Errors detected during MAC address data collection:"
$		write sys$output -
	"---------------------------------------------------"
$		search 'temp_file'_mac -
			"This procedure requires",-
			error,"-f-","-e-","-w-","-s-","-i-"
$		write sys$output -
	"---------------------------------------------------"
$!!		dc_errors = "TRUE"
$	endif
$!
$! Parse the output from the SHOWLAN recursive call to obtain the hardware and
$! physical MAC addresess needed for LAVC$FAILURE_ANALYSIS.
$!
$! First, try to determine the cluster name
$	cluster_name = f$trnlnm("EDIT_LAVC_DESC_CLUSTER")
$	if cluster_name .eqs. ""
$	then	!Cluster name not specified via logical name; try DECnet alias
$		decnet_cluster_alias = f$trnlnm("SYS$CLUSTER_NODE")
$		if decnet_cluster_alias .nes. ""
$		then	!Use DECnet cluster alias as cluster name
$			cluster_name = decnet_cluster_alias - "::"
$		else
$			cluster_name = "the"
$		endif
$	endif
$!
$	close/nolog na
$	open/write na 'temp_file'_na	!Info about nodes / adapters
$	close/nolog log
$	open/read log 'temp_file'_mac	!MAC address info we got from SDA
$!
$	node = "<unknown>"
$	adapter = "<unknown>"
$	hw_addr = "<unknown>"
$	phys_addr = "<unknown>"
$	addr_found = "F"
$       tab = " "
$       tab[0,7] = 9    !Horizontal tab
$! Here's a sample of what we'll be parsing:
$!Node BBC309
$!           -- EIA2 60-07 (SCA) Unit Information 21-NOV-2002 12:02:05 --
$!Hardware address   00-02-A5-6B-23-A2    User transmit FC/AC               ON
$!Physical address   AA-00-04-00-55-06    User receive FC/AC               OFF
$!***************
$!        -- EIA2 60-07 (SCA) Unit Information (cont) 21-NOV-2002 12:02:05 --
$!
$! From a VAX, it looks like this instead:
$!Node BUDDRY
$!        -- FXA1 60-07 (SCA) Unit Information  5-DEC-2002 18:41:19 --
$!802.2 service                    OFF    Hardware address   08-00-2B-9B-76-ED
$!Data chaining                    OFF    Physical address   08-00-2B-9B-76-ED
$!***************
$!      -- FXA1 60-07 (SCA) Counters & Misc Info  5-DEC-2002 18:41:19 --
$!***************
$!     -- FXA1 60-07 (SCA) Multicast Address Info  5-DEC-2002 18:41:19 --
$!802.2 service                    OFF    Hardware address   08-00-2B-9B-76-ED
$!Data chaining                    OFF    Physical address   AA-00-04-00-C5-73
$!
$ mac_loop:
$	read/end=mac_eof log rec
$	if f$extract(0,5,rec) .eqs. "Node "
$	then
$		gosub dump_output	!Dump a record if we have one
$		node = f$element(1," ",rec)	!Nodename
$! See if they specified a description string for this node
$		node_desc = f$trnlnm("EDIT_LAVC_DESC_NODE_''node'")
$		if node_desc .eqs. "" then node_desc = -
    " node of ''cluster_name' cluster; " + f$getsyi("HW_NAME",node)
$		write na ""
$		write na tab,"SYSTEM",tab,node,",",tab,node,",",tab,-
                	"<",node_desc,">"
$		adapter = "<unknown>"	!We don't yet know any adapters on this node
$!!! show symbol rec	!!!
$!!! show symbol node	!!!
$		goto mac_loop
$	endif
$	crec = f$edit(rec,"COMPRESS,TRIM")
$!!! show symbol crec	!!!
$!-- EIA2 60-07 (SCA) Unit Information 21-NOV-2002 12:02:05 --
$! but we want to ignore lines like the following:
$!-- EIA2 60-07 (SCA) Unit Information (cont) 21-NOV-2002 12:02:05 --
$	if f$extract(0,3,crec) .eqs. "-- "
$	then
$	  if f$element(3," ",crec) .eqs. "(SCA)" .and. -
	     f$element(4," ",crec) .eqs. "Unit" .and. -
	     f$element(6," ",crec) .nes. "(cont)"
$	  then	!Found a LAN adapter running SCS
$		gosub dump_output	!Dump a record if we have one
$		adapter = f$extract(0,3,f$element(1," ",crec))
$!!! show symbol adapter	!!!
$		hw_addr = "<unknown>"	!Don't yet know the MAC address(es)
$		phys_addr = "<unknown>"
$		addr_found = "F"	!Pick up LAN addresses now
$		goto mac_loop
$	  else	!Ignore LAN addresses found after later headings
$	    addr_found = "T"	!Stop looking for LAN addresses now
$	  endif
$	endif
$	if (f$locate("Hardware address",crec) .ne. f$length(crec)) .and. -
		(.not. addr_found)
$	then
$		t = f$locate("Hardware address",crec) + -
			f$length("Hardware address") + 1	!Point to MAC address
$		hw_addr = f$element(0," ",f$extract(t,99,crec))
$!!! show symbol hw_addr	!!!
$		goto mac_loop
$	endif

$	if (f$locate("Physical address",crec) .ne. f$length(crec)) .and. -
		(.not. addr_found)
$	then
$		t = f$locate("Physical address",crec) + -
			f$length("Physical address") + 1	!Point to MAC address
$		phys_addr = f$element(0," ",f$extract(t,99,crec))
$!!! show symbol phys_addr	!!!
$		addr_found = "T"	!Stop looking for LAN addresses now
$		goto mac_loop
$	endif
$	goto mac_loop
$!
$ mac_eof:
$	gosub dump_output	!Dump a record if we have one
$	close log
$	delete 'temp_file'_mac;*
$	close na
$	goto mac_done	!Skip over dump_output subroutine
$!
$! Subroutine to dump the code for an adapter
$!
$ dump_output:
$	if adapter .nes. "<unknown>"
$	then
$! See if they specified a description string for this adapter on this node
$	    adapter_desc = -
		f$trnlnm("EDIT_LAVC_DESC_NODE_''node'_ADAPTER_''adapter'")
$	    if adapter_desc .eqs. "" then adapter_desc = -
		"Node " + node + ", LAN adapter device name " + adapter
$! See if they're running DECnet Phase IV and need a DECnet-style MAC address
$! included in addition to the hardware MAC address of the adapter itself
$	    if phys_addr .nes. hw_addr .and. phys_addr .nes. "FF-FF-FF-FF-FF-FF"
$	    then	!Display DECnet Phase IV-style MAC address also
$		write na tab,"LAN_ADP",tab,adapter,"_",node,",  ,  <",-
		    adapter_desc,">,",tab,"<",hw_addr,">, <",phys_addr,">"
$	    else	!Display only hardware MAC address (no DECnet Phase IV)
$		write na tab,"LAN_ADP",tab,adapter,"_",node,",  ,  <",-
		    adapter_desc,">,",tab,"<",hw_addr,">"
$	    endif
$	endif
$	return
$!
$ mac_done:
$!
$!
$! Gather SCS LAN-path info using a recursive call to this procedure (with the
$! SHOW_ALL_CHANNELS parameter) on each cluster node
$!
$	close/nolog com
$	open/write com 'temp_file'_com
$	write com "$ mcr sysman set environment/cluster"
$	write com "set profile/priv=(cmkrnl)"
$	write com "SET TIMEOUT 0:0:30"  !Give up after 30 seconds
$	write com "do @",proc," SHOW_ALL_CHANNELS"	!Call us recursively
$	write com "exit"
$	write com "$ exit"
$	close com
$	@'temp_file'_com/out='temp_file'_paths
$	delete 'temp_file'_com;*
$ 'if_debug' type/page/header 'temp_file'_paths	!!!
$! See if there were any error messages in the output
$	define/user sys$output _NLA0:
$	define/user sys$error _NLA0:
$	search/out=_NLA0: 'temp_file'_paths -
		"LAN SCS communications is disabled",-
		"This procedure requires",-
		error,"-f-","-e-","-w-"
$	if $severity .eq. 1	!1 means string found; 3 means none found
$	then	!Got some errors during data collection
$		write sys$output -
	"Errors detected during SCS LAN path data collection:"
$		write sys$output -
	"----------------------------------------------------"
$		search 'temp_file'_paths "> ----- <"/match=nor
$		write sys$output -
	"----------------------------------------------------"
$!!		dc_errors = "TRUE"
$	endif
$! The SHOW_ALL_CHANNELS section below outputs a
$! "Channel data collection was successful on node xxxx" message
$! when it is able to gather path data on a node.
$! @@@@@  We should check each of these messages against our list of cluster
$! nodes and note any specific nodes where the data collection has failed for
$! any reason.  At present, we depend on a simple $SEARCH instead.  @@@@@
$!
$! Sort nodenames and adapter names in alphabetical order for neatest output
$! (This sorts only the local node and local adapter; the remote nodes and
$! adapters are still unordered.  Note that if we detect failures during
$! data collection and use the remote nodename/adapter name to pick up some
$! of the missing data, the result is that the results may not be alphabetical
$! anymore.)
$	sort/key=(position:2,size:12)/duplicates -
		'temp_file'_paths 'temp_file'_paths_sorted
$	delete 'temp_file'_paths;*
$ 'if_debug' type/page/header 'temp_file'_paths_sorted
$! Read the sorted list of paths and create DCL symbols containing the
$! connectivity matrix info
$	node_list = ","
$	node_cnt = 0
$	close/nolog paths
$	open/read paths 'temp_file'_paths_sorted
$! Read the sorted list of paths and create DCL symbols containing the
$! connectivity matrix info
$	node_list = ","	!List of nodes in the cluster
$	node_cnt = 0	!Count of nodes in the cluster
$	close/nolog paths
$	open/read paths 'temp_file'_paths_sorted
$! Here's an example of what we're parsing:
$! [NCACAA]: EIA <EI_82559> ----- <EI_82559> EIB :[NCACAF]
$! [NCACAA]: EWC <EW_DE500> ----- <EW_DE500> EWC :[NCACAF]
$!  2 <-- POSITION for SORT
$!  123456789.12 <-- SIZE for SORT
$ loop:
$	read/end=eof paths rec
$	if f$extract(0,1,rec) .nes. "[" then goto loop
$	loc_node = f$element(0," ",rec) - "[" - "]" - ":"
$	loc_dev_name = f$element(1," ",rec)
$!!!	loc_dev_type = f$element(2," ",rec) - "<" - ">"
$!!!	rmt_dev_type = f$element(4," ",rec) - "<" - ">"
$	rmt_dev_name = f$element(5," ",rec)
$	rmt_node = f$element(6," ",rec) - "[" - "]" - ":"
$! See if the node is in our list
$	if f$locate(("," + loc_node + ","),node_list) .eq. f$length(node_list)
$	then
$		node_list = node_list + loc_node + ","
$		node_cnt = node_cnt + 1
$	endif
$! Record the path between adapters for the connectivity matrix we're making
$	PATH_'loc_node'_'loc_dev_name'_'rmt_node'_'rmt_dev_name' = "+"
$! Record the path in the reverse direction also, if it's not already noted.
$	if f$type(PATH_'rmt_node'_'rmt_dev_name'_'loc_node'_'loc_dev_name') -
		.eqs. "" then -
		PATH_'rmt_node'_'rmt_dev_name'_'loc_node'_'loc_dev_name' = "+"
$! Now record the adapter, if we don't already have a record of it in the
$! list for the associated node.
$! First, see if the node has an adapter list yet.  If not, create one.
$	if f$type(ADP_'loc_node') .eqs. ""
$	then	!Adapter list needs to be created for this node
$		ADP_'loc_node' = ","
$		ADP_'loc_node'_CNT = 0
$	endif
$! See if the local adapter is in our adapter list for the node
$	if f$locate(("," + loc_dev_name + ","),ADP_'loc_node') -
		.eq. f$length(ADP_'loc_node')
$	then
$		ADP_'loc_node' = ADP_'loc_node' + loc_dev_name + ","
$		ADP_'loc_node'_CNT = ADP_'loc_node'_CNT + 1
$	endif
$	goto loop
$ eof:
$	close paths
$! If we had data-collection errors, we may be missing data from some nodes.
$! In that case, we'll make another pass through the list, picking up some
$! extra data by looking at each indicated path from the opposite end -- that
$! of the remote node and remote adapter.  We'll still be missing data on
$! connections among the nodes with data-collection errors, but at least we'll
$! be able to see paths from the nodes we do know about to the problem nodes.
$!
$! (We do this scan as a second pass because if we try to pick the remote data
$! up in the initial pass, it greatly messes up our alphabetical ordering of
$! nodenames and adapter names.  By adding missing data at the end, less
$! non-alphabetic ordering of node and adapter names results.)
$!!	if .not. dc_errors then goto end_pass2	!Skip if we got all the data
$!!! We'll go ahead and do the second pass all the time, just in case an
$!!! error in data collection slips past our detection code.
$	dc_err_nodes = ","
$	open/read paths 'temp_file'_paths_sorted
$! As a reminder, here's an example of what we're parsing:
$! [NCACAA]: EIA <EI_82559> ----- <EI_82559> EIB :[NCACAF]
$! [NCACAA]: EWC <EW_DE500> ----- <EW_DE500> EWC :[NCACAF]
$ loop2:
$	read/end=eof2 paths rec
$	if f$extract(0,1,rec) .nes. "[" then goto loop2
$	loc_node = f$element(0," ",rec) - "[" - "]" - ":"
$	loc_dev_name = f$element(1," ",rec)
$!!!	loc_dev_type = f$element(2," ",rec) - "<" - ">"
$!!!	rmt_dev_type = f$element(4," ",rec) - "<" - ">"
$	rmt_dev_name = f$element(5," ",rec)
$	rmt_node = f$element(6," ",rec) - "[" - "]" - ":"
$! See if the node is in our list
$	if f$locate(("," + rmt_node + ","),node_list) .eq. f$length(node_list)
$	then	!Not there -- add it
$		node_list = node_list + rmt_node + ","
$		node_cnt = node_cnt + 1
$! Also add it to our list of nodes for which data is apparently missing
$		dc_err_nodes = dc_err_nodes + rmt_node + ","
$	endif
$! Now record the adapter, if we don't already have a record of it in the
$! list for the associated node.
$! First, see if the node has an adapter list yet.  If not, create one.
$	if f$type(ADP_'rmt_node') .eqs. ""
$	then	!Adapter list needs to be created for this node
$		ADP_'rmt_node' = ","
$		ADP_'rmt_node'_CNT = 0
$	endif
$! See if the remote adapter is in our adapter list for the remote node
$	if f$locate(("," + rmt_dev_name + ","),ADP_'rmt_node') -
		.eq. f$length(ADP_'rmt_node')
$	then
$		ADP_'rmt_node' = ADP_'rmt_node' + rmt_dev_name + ","
$		ADP_'rmt_node'_CNT = ADP_'rmt_node'_CNT + 1
$	endif
$	goto loop2
$ eof2:
$	close paths
$!! end_pass2:
$	delete 'temp_file'_paths_sorted;*
$ 'if_debug' show symbol node_cnt	!!!
$ 'if_debug' show symbol node_list	!!!
$ 'if_debug' show symbol dc_err_nodes	!!!
$ if dc_err_nodes .nes. "," then -
	write sys$output "*** Data appears to be missing for node(s): ",-
		f$extract(1,f$length(dc_err_nodes)-2,dc_err_nodes)," ***"
$ 'if_debug' show symbol ADP_*	!!!
$ 'if_debug' show symbol PATH_*	!!!
$!
$! Now we know about all the adapters on all nodes, and the connectivity 
$! between all possible pairs of adapters.
$!
$! Based on this data, we determine the sets of network "segments" that must
$! exist to interconnect the adapters.
$!
$! We make some simplifying assumptions, such as if there is connectivity
$! between adapter A and adapter B, and adapter A and adapter C, that
$! adapters A, B, and C must all be on the same extended LAN (which we call
$! a Network Segment here), and assume there must also be connectivity between
$! adapters B and C.
$!
$! The basic algorithm we use is:
$!	Starting with the first node, and
$!	Starting with the first adapter on that node,
$!	Find out all the other adapters, on all nodes
$!		which can talk to that first adapter.  Assume all these
$!		adapters are connected to Network Segment 1.
$!	Next we find the next so-far-unconnected adapter, and assign this
$!		to Network Segment 2, similarly finding all the adapters that 
$!		it can talk to and assigning all these adapters to Network
$!		Segment 2 as well.
$!	We go through all nodes and adapters creating network segments in this
$!	manner.  We count the number of network segments we create as we go.
$!
$	segment_cnt = 0	!Count of segments we've created
$! Process each and every node in turn:
$	n = 1	!'N' is the index of the node
$ loop_node:
$	node = f$element(n,",",node_list)
$! Now, for each adapter on the local node:
$	a = 1	!'A' is the index into the adapters on node 'N'
$ loop_adp:
$	dev_name = f$element(a,",",ADP_'node')
$! Now see if this adapter has a connection to a segment yet
$	if f$type(SEG_'node'_'dev_name') .eqs. ""
$	then	!This adapter hasn't been connected to a segment yet
$		segment_cnt = segment_cnt + 1	!Create a new segment
$		SEG_'node'_'dev_name' = segment_cnt	!Connect adapter to new segment
$!!! write sys$output -
$!_!!!    "Creating new Network Segment ''segment_cnt' for ''node'/''dev_name'"
$! Now connect to the same segment all adapters that this adapter can talk to
$		n1 = 1	!N1 is the index of one node
$ loop_node1:
$		node1 = f$element(n1,",",node_list)
$! Now, for each adapter on Node 1:
$		a1 = 1	!A1 is the index into the adapters on node 'N1'
$ loop_adp1:
$		dev_name1 = f$element(a1,",",ADP_'node1')
$! Now, for each of the other nodes
$		n2 = 1	!N2 is the index of the other node
$ loop_node2:
$		node2 = f$element(n2,",",node_list)
$! Now, for each adapter on the other node:
$		a2 = 1		!A2 is the index into the adapters on node 'N2'
$ loop_adp2:
$		dev_name2 = f$element(a2,",",ADP_'node2')
$		t1 = -	!We have connectivity to Node1/Dev_Name1
	(f$type(PATH_'node1'_'dev_name1'_'node'_'dev_name') .nes. "")
$		t2 = -	!We have connectivity to Node2/Dev_Name2
	(f$type(PATH_'node'_'dev_name'_'node2'_'dev_name2') .nes. "")
$		if t1
$		then
$			if f$type(SEG_'node1'_'dev_name1') .eqs. ""
$			then
$!!! write sys$output "Connecting ''node1'/''dev_name1' to Network Segment ",segment_cnt
$				SEG_'node1'_'dev_name1' = segment_cnt
$			else	!This adapter was already assigned to a segment
$				if SEG_'node1'_'dev_name1' .ne. segment_cnt
$				then
$					write sys$output -
"*** Trying to assign ''node1'/''dev_name1' to Network Segment ''segment_cnt'"
$					write sys$output -
"    but it's already connected to Network Segment ",SEG_'node1'_'dev_name1',"."
$				endif
$			endif
$		endif
$		if t2
$		then
$			if f$type(SEG_'node2'_'dev_name2') .eqs. ""
$			then
$ !!! write sys$output "Connecting ''node2'/''dev_name2' to Network Segment ",segment_cnt
$				SEG_'node2'_'dev_name2' = segment_cnt
$			else	!This adapter was already assigned to a segment
$				if SEG_'node2'_'dev_name2' .ne. segment_cnt
$				then
$					write sys$output -
"*** Trying to assign ''node2'/''dev_name2' to Network Segment ''segment_cnt'
$					write sys$output -
"    but it's already connected to Network Segment ",SEG_'node2'_'dev_name2',"."
$				endif
$			endif
$		endif
$! Repeat for each adapter on Node2
$		a2 = a2 + 1
$		if a2 .le. ADP_'node2'_CNT then goto loop_adp2
$! Repeat for each possible Node2
$		n2 = n2 + 1
$		if n2 .le. node_cnt then goto loop_node2
$! Repeat for each adapter on Node1
$		a1 = a1 + 1
$		if a1 .le. ADP_'node1'_CNT then goto loop_adp1
$! Repeat for each possible Node1
$		n1 = n1 + 1
$! See if there are more nodes to process
$		if n1 .le. node_cnt then goto loop_node1
$! We've finished noting all adapters which are connected to this segment
$!	else
$!		!This adapter has been assigned to a network segment already
$	endif
$! Repeat for each local adapter
$	a = a + 1
$	if a .le. ADP_'node'_CNT then goto loop_adp
$! Repeat for each local node
$	n = n + 1
$! See if there are more nodes to process
$	if n .le. node_cnt then goto loop_node
$! Now we're ready to start generating some output.  We generate three
$! streams at once:
$!   1. ASCII-Art drawing of LAN configuration
$!	(This will be used for Edit 2 in LAVC$FAILURE_ANALYSIS.MAR)
$!   2. VLAN declarations
$!	(This will be used for Edit 4 in LAVC$FAILURE_ANALYSIS.MAR)
$!   3. CONNECTION Macro calls
$!	(This will be used for Edit 5 in LAVC$FAILURE_ANALYSIS.MAR)
$	close/nolog art
$	open/write art 'temp_file'_art	!LAN drawing art
$	close/nolog vln
$	open/write vln 'temp_file'_vln	!VLANs
$	close/nolog con
$	open/write con 'temp_file'_con	!Connections
$!!	tab = " "
$!!	tab[0,7] = 9	!ASCII Horizontal Tab character
$! The list of VLANs is easiest, so we'll do that first.  This looks like:
$!	VLAN	VLAN_1, ,  <VLAN #1>
$!	VLAN	VLAN_2, ,  <VLAN #2>
$! etc., (unless the user have overridden the default descriptions)
$	i = 0
$ vln_loop:
$	i = i + 1
$	vlan_desc = f$trnlnm("EDIT_LAVC_DESC_VLAN_''i'")
$	if vlan_desc .eqs. "" then vlan_desc = f$fao("VLAN #!UL",i)
$	write vln tab,"VLAN",tab,"VLAN_",i,", ",tab,",",tab,"<",vlan_desc,">"
$	if i .lt. segment_cnt then goto vln_loop
$	close vln
$!
$! Create a drawing of network segments and adapter connections of the
$! following form:
$!
$! Node_1 Ad1---*
$!              | 
$!        Ad2---)--*
$!              |  |
$!              |  |
$! Node_2 Ad1---*  |
$!              |  |
$!        Ad2---)--*
$!              |  |
$!        Ad3---)--)--*
$!              |  |  |
$!              |  |  |
$! Node_3 Ad1---)--)--*
$!              |  |  |
$!        Ad2---)--*  |
$!              |  |  |
$!        Ad3---*  |  |
$!              |  |  |
$!              |  |  |
$!       VLAN:  1  2  3
$!
$!  where "*" represents a connection, and ")" indicates NO connection.
$!
$!	                Nodenm Adp
$	segment_line = "          "
$	segments_shown = 0
$!
$! Begin generating the main area of the drawing a line at a time.
$!
$	write art ";"
$! Process each and every node in turn:
$	i = 1	!I is the index of the node at the left-hand side, for this row
$ loop_loc_node:
$	loc_node = f$element(i,",",node_list)
$! See if this node is one for which we believe there were data-collection errors
$	node_error_flag = ""
$	if f$locate(","+loc_node+",",dc_err_nodes) .ne. f$length(dc_err_nodes) -
		then node_error_flag = -
		  " (Data collection appeared incomplete for node ''loc_node')"
$	write con ";"	!Some comments showing the start of connection data for this node
$	write con "; ",loc_node,node_error_flag
$	write con ";"
$	node_field = loc_node	!Set up to put nodename into the left-hand column
$! Now, for each adapter on the local node:
$	j = 1	!J is the index into the adapters on node 'i'
$ loop_loc_adp:
$	loc_dev_name = f$element(j,",",ADP_'loc_node')
$! Create a connection from this host node to this adapter
$	write con tab,"CONNECTION",tab,loc_node,tab,loc_dev_name,"_",loc_node
$! Also create a connection from this adapter to its network segment (VLAN)
$	write con tab,"CONNECTION",tab,tab,loc_dev_name,"_",loc_node,tab,-
			"VLAN_",SEG_'loc_node'_'loc_dev_name'
$! If we aren't already showing this segment, start showing it now
$ seg_show_loop:
$	if SEG_'loc_node'_'loc_dev_name' .gt. segments_shown
$	then
$		segment_line = segment_line + "  |"
$		segments_shown = segments_shown + 1
$		goto seg_show_loop
$	endif
$! Build the line showing the connection from this adapter to a network segment
$	k = 0
$	net_connects = ""
$ net_loop:
$	k = k + 1
$	if SEG_'loc_node'_'loc_dev_name' .eq. k
$	then	!Indicate a connection to this segment
$		net_connects = net_connects + "--*"
$	else	!Indicate there is NO connection to this segment
$		if k .gt. SEG_'loc_node'_'loc_dev_name'
$		then
$			net_connects = net_connects + "  |"
$		else	!Haven't connected to our network segment yet
$			net_connects = net_connects + "--)"
$		endif
$	endif
$	if k .lt. segments_shown then goto net_loop
$!
$	write art "; ",f$fao("!6AS !3AS",-
		node_field,loc_dev_name),net_connects,node_error_flag
$	write art "; ",segment_line
$	node_field = " "	!Blank out nodename for succeeding adapters
$!				!on the same node
$	node_error_flag = ""	!And don't repeat warning about data-collection
$!				!errors
$!
$! Repeat for each local adapter
$	j = j + 1
$	if j .le. ADP_'loc_node'_CNT then goto loop_loc_adp
$! Generate an extra line to separate nodes from each other
$	write art "; ",segment_line
$! Repeat the process for each local node
$	i = i + 1
$! See if there are more nodes to process
$	if i .le. node_cnt then goto loop_loc_node
$!
$	write art "; ",segment_line
$!!!	last_line = "  Segment:"
$	last_line = "     VLAN:"
$	k = 0
$! Display the last line, indicating the segment numbers.  For example:
$!    Segment:  1  2  3
$ net_seg_loop:
$	k = k + 1
$	last_line = last_line + f$fao("!3UL",k)
$	if k .lt. segments_shown then goto net_seg_loop
$	write art "; ",last_line
$	write art ";"
$	close art
$	close con
$!
$! All the output has been created.  Merge the various streams into the source.
$!
$	open/read in sys$examples:lavc$failure_analysis.mar
$	open/write out 'temp_file'_all	!Create in default directory
$! Read the source code until we hit the line:
$!;       *** Start edits here ***
$ pre_loop:
$	read/end=premature_eof in rec
$	crec = f$edit(rec,"COMPRESS,TRIM")
$	if crec .nes. "; *** Start edits here ***"
$	then
$		write out rec
$		goto pre_loop
$	endif
$	write out rec	! Include "Start edits here" line
$!
$! Provide some informative comments in the generated code as to where it
$! came from
$	write out ""	!Blank line
$	write out -
";---------------------------------------------------------------------------"
$	write out -
"; The site-specific code below was generated ",f$time()
$	write out -
"; with the assistance of EDIT_LAVC.COM ",edit_lavc_version
$	write out -
"; written by Keith Parris, parris@encompasserve.org or keithparris@yahoo.com"
$	write out -
"; or keith.parris@hp.com.  The latest version should be accessible via"
$	write out -
"; http://encompasserve.org/~parris/ or http://www.geocities.com/keithparris/"
$	write out -
";---------------------------------------------------------------------------"
$!
$! Generate "Edit 1" code
$!
$	write out ""	!Blank line
$	write out ";",tab,"Edit 1."
$	write out ";"
$	write out ";",tab,tab,-
		"Define the hardware components needed to describe"
$	write out ";",tab,tab,"the physical configuration."
$	write out ";"
$	write out ""	!Blank line
$	write out tab,"NEW_COMPONENT",tab,"SYSTEM",tab,tab,"NODE"
$	write out tab,"NEW_COMPONENT",tab,"LAN_ADP",tab,tab,"ADAPTER"
$	write out tab,"NEW_COMPONENT",tab,"VLAN",tab,tab,"COMPONENT",tab,-
		";Virtual LAN (VLAN) or Extended LAN"
$	write out ""	!Blank line
$	ff = " "
$	ff[0,7] = 12	!Form-feed
$	write out ff
$!
$! Generate "Edit 2" code
$!
$ 'if_debug' type/page/header 'temp_file'_art	!ASCII Art drawing:	Edit 2
$	write out ";",tab,"Edit 2."
$	write out ";"
$	write out ";",tab,tab,-
		"Diagram of the cluster and SCS LAN configuration."
$	write out ";"
$	close out
$	append 'temp_file'_art 'temp_file'_all
$	open/append out 'temp_file'_all
$	write out ";"
$!
$! Generate "Edit 3" code
$!
$ 'if_debug' type/page/header 'temp_file'_na	!Nodes/adapters:	Edit 3
$	write out ";",tab,"Edit 3."
$	write out ";"
$	write out ";",tab,tab,-
		"Label all the systems and LAN adapters."
$	write out ";"
$	write out ";",tab,tab,"Label",tab," Node",tab,tab,-
		"      Description",tab,tab,tab,"    LAN HW Addr",tab,-
		"         DECnet Addr"
$	write out ";",tab,tab,"-----",tab,"------",tab,-
		"------------------------------------------",tab,-
		"-------------------  -------------------"
$	close out
$	append 'temp_file'_na 'temp_file'_all
$	open/append out 'temp_file'_all
$	write out ""	!Blank line
$!
$! Generate "Edit 4" code
$!
$ 'if_debug' type/page/header 'temp_file'_vln	!VLANs:			Edit 4
$	write out ";",tab,"Edit 4."
$	write out ";"
$	write out ";",tab,tab,-
		"Label each of the other network components."
$	write out ";"
$	write out ""	!Blank line
$	close out
$	append 'temp_file'_vln 'temp_file'_all
$	open/append out 'temp_file'_all
$	write out ff
$!
$! Generate "Edit 5" code
$!
$ 'if_debug' type/page/header 'temp_file'_con	!Connections:		Edit 5
$	write out ";",tab,"Edit 5."
$	write out ";"
$	write out ";",tab,tab,-
		"Describe the network connections."
$	write out ";"
$	write out ""	!Blank line
$	close out
$	append 'temp_file'_con 'temp_file'_all
$	open/append out 'temp_file'_all
$	write out ff
$	write out tab,".PAGE"
$	write out tab,".SBTTL",tab,"Define the Local Data."
$	write out ""	!Blank line
$! Now, skip over the Edits in the template from SYS$EXAMPLES:
$ mid_loop:
$	read/end=premature_eof in rec
$	crec = f$edit(rec,"COMPRESS,TRIM")
$	if crec .nes. "; *** End of edits ***" then goto mid_loop
$!
$	write out rec	! Include "End of edits" line
$! Now include all the rest of the code from the tempate in SYS$EXAMPLES:
$ post_loop:
$	read/end=post_eof in rec
$	write out rec
$	goto post_loop
$! End of the template program
$ post_eof:
$	close in
$	close out
$! Now that it is complete, rename the output file to its proper name
$	rename 'temp_file'_all; lavc$failure_analysis.mar;
$	goto delete_temp_files
$ premature_eof:
$	close in
$	close out
$	write sys$output "*** Error parsing SYS$EXAMPLES:LAVC$FAILURE_ANALYSIS.MAR ***"
$	write sys$output "*** Procedure has failed. ***"
$ delete_temp_files:
$! Clean up after ourselves by getting rid of the temporary files we used
$	delete 'temp_file'_na;*,-
	       'temp_file'_art;*,-
	       'temp_file'_vln;*,-
	       'temp_file'_con;*
$	if f$search("''temp_file'_all;*") .nes. "" then -
		delete 'temp_file'_all;*
$! All done.  Clean up and exit.
$ cleanup:
$! If we turned on OPER priv, turn it off again
$	if OPER_TURNED_ON then set process/privilege=(noOPER)
$	exit
$!
$!-----------------------------------------------------------------------------
$!
$ SHOW_ALL_CHANNELS:
$!
$ if_standalone = "!"	!"" for use as a standalone procedure; "!" for use
$!			!within SHOW_PATHS.COM or SHOW_SEGMENTS.COM.
$!
$! SHOW_ALL_CHANNELS.COM
$!
$! Show all the active LAN channels (paths) seen by PEDRIVER to each remote node
$!
$! Based on SHOW_PREF_PATH.COM, SHOW_ALL_PATHS.COM
$!						V1.0 3/2001
$! Author: Keith Parris  parris@encompasserve.org or keithparris@yahoo.com
$! http://www.geocities.com/keithparris/ or http://encompasserve.org/~parris/
$! Modified for 7.3-1    KP 10/8/2002
$!	Output format changed for SDA> SHOW PORTS/VC/NODE=xxx
$! Improved		 KP 12/4/2002
$!	VAX SDA apparently doesn't support /NOINDEX qualifer to SET OUTPUT command
$!	Modified to also show paths between different LAN adapters on the same
$!	node, to fix problems when used for SHOW_SEGMENTS.COM, with its need to
$!	map adapters to network segments.
$!	Improved error-checking and privilege handling
$!
$	if_debug = "!"	!Use "!" for normal operations, and "" for debugging
$! Check to see if SCS communications over the LAN is even enabled on this node.
$! If not, SDA> SHOW PORTS will not define the PE_PDT symbol and we can't
$! gather any data.
$	if (.not. f$getdvi("PEA0","EXISTS")) .or. -
	   (.not. f$getsyi("NISCS_LOAD_PEA0"))
$	then
$	    write sys$output "LAN SCS communications is disabled on node ",-
			f$getsyi("NODENAME"),"."
$!	    write sys$output "NISCS_LOAD_PEA0 parameter value is ",-
$!_!			f$getsyi("NISCS_LOAD_PEA0"),"."
$		exit
$	endif
$!
$	data_found = "F"	!Assume we failed until proven otherwise
$! Check for the privileges required to run SDA
$	auth_privs = "," + f$getjpi("","AUTHPRIV") + ","
$	cur_privs  = "," + f$getjpi("","CURPRIV")  + ","
$! Remember whether or not we had to elevate privileges
$	ELEVATED_PRIVS = "F"	!Assume process already had CMKRNL privilege turned on
$	remove_privs = ""
$! Check for CMKRNL privilege and turn it on if need be
$	if f$locate(",CMKRNL,",cur_privs) .eq. f$length(cur_privs)
$	then	!CMKRNL privilege is not presently turned on
$		if f$locate(",CMKRNL,",auth_privs) .ne. f$length(auth_privs) .or. -
		   f$locate(",SETPRV,",auth_privs) .ne. f$length(auth_privs)
$		then
$			set process/privilege=(CMKRNL)
$			remove_privs = remove_privs + ",noCMKRNL"
$			ELEVATED_PRIVS = "T"
$		else
$			write sys$output -
		    "This procedure requires CMKRNL privilege to run SDA."
$			exit
$		endif
$	endif
$! Check for READALL privilege and turn it on if possible
$	if f$locate(",READALL,",cur_privs) .eq. f$length(cur_privs)
$	then	!READALL privilege is not presently turned on
$		if f$locate(",READALL,",auth_privs) .ne. f$length(auth_privs) .or. -
		   f$locate(",SETPRV,",auth_privs) .ne. f$length(auth_privs)
$		then
$			set process/privilege=(READALL)
$			remove_privs = remove_privs + ",noREADALL"
$			ELEVATED_PRIVS = "T"
$!		else
$! Most systems seem to run SDA with only CMKRNL -- I've only hit a few
$! that apparently have protection problems or something and require READALL.
$! If the user doesn't have READALL, we'll just try SDA anyway without it.
$!			write sys$output -
$!_!		    "This procedure requires READALL privilege to run SDA."
$!			exit
$		endif
$	endif
$	remove_privs = remove_privs - ","	!Remove leading comma
$! Make temporary filenames unique by PID
$!In captive mode, use temporary filenames from main procedure
$ 'if_standalone' pid = f$getjpi("","PID")
$ 'if_standalone' temp_file := show_all_channels_temp_'pid'.temp_file
$	delete := delete
$! First, find out the names of all the nodes in the cluster
$	this_node = f$getsyi("NODENAME")	!This is where we're running
$ 'if_debug' show symbol this_node
$	nodecount = 0
$	context = ""
$ loop:
$	csid = f$csid(context)	!Get the CSID of the next cluster member
$	if csid .nes. ""
$	then
$	    nodecount = nodecount + 1	!1 more node
$	    nodename_'nodecount' = f$getsyi("NODENAME",,csid)
$ 'if_debug' show symbol nodename_'nodecount'
$	    goto loop
$	endif
$ 'if_debug' show symbol nodecount
$! Next, use SDA to find out the channel addresses of all the LAN paths
$! to each node
$	close/nolog com
$	open/write com 'temp_file'_com
$!!!!	write com "$ DEFINE/USER SYS$OUTPUT _NLA0:"
$!!!!	write com "$ DEFINE/USER SYS$ERROR _NLA0:"
$!!	write com "$ DEFINE/USER SYS$OUTPUT ",temp_file,"_DISCARD"
$!!	write com "$ DEFINE/USER SYS$ERROR ",temp_file,"_DISCARD"
$	write com "$ ANALYZE/SYSTEM"
$! To abbreviate the output from SDA and reduce the amount of text we have
$! to parse, eliminate the index if possible (some SDA versions don't have
$! the /NOINDEX qualifer on the SET OUTPUT command, and I'm not sure exactly
$! which ones, but we'll try our best here).
$	out_qual = "/NOHEADER/NOINDEX"
$	arch = f$getsyi("ARCH_NAME")
$	if arch .eqs. "VAX"
$	then	!VAX
$		out_qual = ""	!No /INDEX or /HEADER on VAX
$	else	!Alpha and I64
$		vers = f$extract(1,99,f$getsyi("VERSION"))
$		if vers .lts. "7.3" then out_qual = "/NOINDEX"	!No /HEADER prior to 7.3
$		if vers .lts. "7.0" then out_qual = ""
$	endif
$! We'll discard the output from the bare SDA> SHOW PORTS command we do first;
$! we only need it to define the PE_PDT symbol
$!!	write com "SET OUTPUT _NLA0:"
$!! We got an error from SDA on some versions using "_NLA0:" as the output
$!! filespec., so we'll use an actual file and delete it later.
$	write com "SET OUTPUT ''temp_file'_DISCARD"
$	write com "SHOW PORTS"	!To set symbols for use later
$	write com "SET OUTPUT''out_qual' ",temp_file,"_LIST"
$	i = 0
$ loop2:
$	i = i + 1
$!(Don't bother to look up a path to this node itself)
$!! Actually, we DO need to look up paths to this node itself.  We'll ignore
$!! the loopback path and path from an adapter to itself, but we will need to
$!! know (for SHOW_PATHS.COM and SHOW_SEGMENTS.COM) whenever there is a path
$!! between two different LAN adapters on the same node, because that indicates
$!! that they are connected to the same VLAN or extended LAN or network segment,
$!! however you want to phrase it.
$!!	if nodename_'i' .nes. this_node then-
$		write com "SHOW PORT/ADDRESS=PE_PDT/VC/NODE=",nodename_'i'
$	if i .lt. nodecount then goto loop2
$	write com "EXIT"
$	write com "$ EXIT"
$	close com
$ 'if_debug' type/page/header 'temp_file'_com
$	@'temp_file'_com/out='temp_file'_out
$	if f$search("''temp_file'_DISCARD") .nes. "" then -
		delete 'temp_file'_discard;*
$	if f$search("''temp_file'_list") .eqs. ""
$	then	!SDA must have failed to write the file
$		write sys$output -
    "*** Error: SDA> SHOW PORT output failure on node ",this_node," ***"
$		write sys$output " "
$		write sys$output "  --------------------------------------"
$		write sys$output "  |||   Output from procedure was:   |||"
$		write sys$output "  vvv                                vvv"
$		type 'temp_file'_out;
$		write sys$output "  ^^^                                ^^^"
$		write sys$output "  |||   End of procedure output.     |||"
$		write sys$output "  --------------------------------------"
$		write sys$output " "
$		write sys$output -
    "  ------------------------------------------------------------"
$		write sys$output -
    "  |||   Procedure with SDA commands being attempted was:   |||"
$		write sys$output -
    "  vvv                                                      vvv"
$		type 'temp_file'_com;
$		write sys$output -
    "  ^^^                                                      ^^^"
$		write sys$output -
    "  |||   End of SDA command file.                           |||"
$		write sys$output -
    "  ------------------------------------------------------------"
$		write sys$output " "
$		delete 'temp_file'_com;*	!Clean up
$		delete 'temp_file'_out;*
$		goto sca_cleanup
$	endif
$ 'if_debug' type/page/header 'temp_file'_list
$!Suppress "%SEARCH-I-NOMATCHES, no strings matched
$! and      %SEARCH-I-NULLFILE, file xxxxxx contains no records
$	define/user sys$output _NLA0:
$	define/user sys$error _NLA0:
$	search 'temp_file'_list "Preferred","Active","path,open"-
		/out='temp_file'_channels
$	search_severity = $severity
$ 'if_debug' show symbol search_severity
$	if search_severity .eq. 3	!No channel data found
$	then	!SDA must have failed to write anything useful to the file
$		write sys$output -
    "*** Error: SDA> SHOW PORT output deficient on node ",this_node," ***"
$		write sys$output " "
$		write sys$output "  --------------------------------------"
$		write sys$output "  |||   Output from procedure was:   |||"
$		write sys$output "  vvv                                vvv"
$		type 'temp_file'_out;
$		write sys$output "  ^^^                                ^^^"
$		write sys$output "  |||   End of procedure output.     |||"
$		write sys$output "  --------------------------------------"
$		write sys$output " "
$		write sys$output -
    "  ------------------------------------------------------------"
$		write sys$output -
    "  |||   Procedure with SDA commands being attempted was:   |||"
$		write sys$output -
    "  vvv                                                      vvv"
$		type 'temp_file'_com;
$		write sys$output -
    "  ^^^                                                      ^^^"
$		write sys$output -
    "  |||   End of SDA command file.                           |||"
$		write sys$output -
    "  ------------------------------------------------------------"
$		write sys$output " "
$		write sys$output "  --------------------------------"
$		write sys$output "  |||   Output from SDA was:   |||"
$		write sys$output "  vvv                          vvv"
$		type 'temp_file'_list
$		write sys$output "  ^^^                          ^^^"
$		write sys$output "  |||   End of SDA output.     |||"
$		write sys$output "  --------------------------------"
$		write sys$output " "
$		delete 'temp_file'_com;*
$		delete 'temp_file'_out;*
$		delete 'temp_file'_list;*
$		delete 'temp_file'_channels;*	!Clean up
$		goto sca_cleanup
$	endif
$	delete 'temp_file'_list;*	!!!
$	delete 'temp_file'_out;*
$	delete 'temp_file'_com;*	!!!
$ 'if_debug' type/page/header 'temp_file'_channels
$	close/nolog chan
$! Now use SDA again to find out the "bus" (LAN adapter) which is currently
$! used in the path to each node
$	open/write com 'temp_file'_com2
$!!!!	write com "$ DEFINE/USER SYS$OUTPUT _NLA0:"
$!!!!	write com "$ DEFINE/USER SYS$ERROR _NLA0:"
$!!	write com "$ DEFINE/USER SYS$OUTPUT ",temp_file,"_DISCARD"
$!!	write com "$ DEFINE/USER SYS$ERROR ",temp_file,"_DISCARD"
$	write com "$ ANALYZE/SYSTEM"
$	write com "SET OUTPUT''out_qual' ",temp_file,"_LIST2"
$	open/read chan 'temp_file'_channels
$!
$! Pick up addresses of data structures for all channels that are working.
$! Here's what we're parsing:
$!
$!Address     Type    Xmt Time Size Preferred    Best        Last State Change
$!--------  --------- -------- ---- ---------  --------   -----------------------
$!81764F00  Preferred 00C59486 1412       289         4   28-FEB-2001 15:25:16.19
$!81764E00  Active    00C59486 1412         3       201   28-FEB-2001 15:25:15.83
$!
$! For 7.3-1:
$!Address   LCL/RMT   State      ECS   Size  AvgRTT(us)  Last State Change
$!--------  -------  --------   -----  ----  ----------  -----------------------
$!812D8540  EWA/EWA  path,open  T/P/S  1426     24220.9   7-OCT-2002 21:36:57.12
$!812B9340  EWB/EWB  path,open  T/P/F  1426       244.1   7-OCT-2002 21:24:40.58
$ loop3:
$	read/end=eof3 chan rec
$	rec = f$edit(rec,"COMPRESS,TRIM")
$!Try the 7.3-1 variant first
$	state = f$element(2," ",rec)
$	if state .eqs. "path,open" then goto got_chan
$	path_type = f$element(1," ",rec)
$	if path_type .nes. "Preferred" .and. path_type .nes. "Active" then -
		goto loop3
$ got_chan:
$!! show symbol rec
$	chan_addr = f$element(0," ",rec)
$!! We're looking for an 8-digit hexadecimal address
$!!	if f$length(chan_addr) .ne. 8 then goto loop3
$	write com "SHOW PORT/CHANNEL=",chan_addr," !"	!,path_type
$	goto loop3
$ eof3:
$	close chan
$	delete 'temp_file'_channels;*	!!!
$	close com
$ 'if_debug' type/page/header 'temp_file'_com2
$	@'temp_file'_com2/out='temp_file'_out2
$	if f$search("''temp_file'_list2") .eqs. ""
$	then	!SDA must have failed to write the file
$		write sys$output -
    "*** Error: SDA> SHOW CHANNEL output failure on node ",this_node," ***"
$		write sys$output " "
$		write sys$output "  --------------------------------------"
$		write sys$output "  |||   Output from procedure was:   |||"
$		write sys$output "  vvv                                vvv"
$		type 'temp_file'_out2;
$		write sys$output "  ^^^                                ^^^"
$		write sys$output "  |||   End of procedure output.     |||"
$		write sys$output "  --------------------------------------"
$		write sys$output " "
$		write sys$output -
    "  ------------------------------------------------------------"
$		write sys$output -
    "  |||   Procedure with SDA commands being attempted was:   |||"
$		write sys$output -
    "  vvv                                                      vvv"
$		type 'temp_file'_com2;
$		write sys$output -
    "  ^^^                                                      ^^^"
$		write sys$output -
    "  |||   End of SDA command file.                           |||"
$		write sys$output -
    "  ------------------------------------------------------------"
$		write sys$output " "
$		delete 'temp_file'_com2;*	!Clean up
$		delete 'temp_file'_out2;*
$		goto sca_cleanup
$	endif
$ 'if_debug' type/page/header 'temp_file'_list2
$!
$! Show the user what we found
$!
$!Suppress "%SEARCH-I-NOMATCHES, no strings matched
$! and      %SEARCH-I-NULLFILE, file xxxxxx contains no records
$	define/user sys$output _NLA0:
$	define/user sys$error _NLA0:
$	search 'temp_file'_list2 -
		"PEDRIVER Channel","Lcl Device:","Rmt Device:"-
		/out='temp_file'_list3
$	search_severity = $severity
$ 'if_debug' show symbol search_severity
$	if search_severity .eq. 3
$	then	!SDA must have failed to write anything useful to the file
$		write sys$output -
    "*** Error: SDA> SHOW CHANNEL output deficient on node ",this_node," ***"
$		write sys$output " "
$		write sys$output "  --------------------------------------"
$		write sys$output "  |||   Output from procedure was:   |||"
$		write sys$output "  vvv                                vvv"
$		type 'temp_file'_out2;
$		write sys$output "  ^^^                                ^^^"
$		write sys$output "  |||   End of procedure output.     |||"
$		write sys$output "  --------------------------------------"
$		write sys$output " "
$		write sys$output -
    "  ------------------------------------------------------------"
$		write sys$output -
    "  |||   Procedure with SDA commands being attempted was:   |||"
$		write sys$output -
    "  vvv                                                      vvv"
$		type 'temp_file'_com2;
$		write sys$output -
    "  ^^^                                                      ^^^"
$		write sys$output -
    "  |||   End of SDA command file.                           |||"
$		write sys$output -
    "  ------------------------------------------------------------"
$		write sys$output " "
$		write sys$output "  --------------------------------"
$		write sys$output "  |||   Output from SDA was:   |||"
$		write sys$output "  vvv                          vvv"
$		type 'temp_file'_list2
$		write sys$output "  ^^^                          ^^^"
$		write sys$output "  |||   End of SDA output.     |||"
$		write sys$output "  --------------------------------"
$		write sys$output " "
$		delete 'temp_file'_list2;*	!Clean up
$		delete 'temp_file'_com2;*
$		delete 'temp_file'_out2;*
$		goto sca_cleanup
$	endif
$	delete 'temp_file'_list2;*	!!!
$	delete 'temp_file'_com2;*	!!!
$	delete 'temp_file'_out2;*
$ 'if_debug' type/page/header 'temp_file'_list3
$	close/nolog tmp
$	open/read tmp  'temp_file'_list3
$! Here's an example of what we're parsing in loop 4:
$! -- PEDRIVER Channel (CH:81A71D40) for Virtual Circuit (VC:81A5FA40) NCACAB --
$!BUS: 819CB580  (EWB)  Lcl Device: EW_DEGPA  Lcl LAN Address: 00-60-6D-21-32-B4
$!Rmt Name: EWB         Rmt Device: EW_DEGPA  Rmt LAN Address: 00-60-6D-21-32-C0
$! For 7.3-1:
$! -- PEDRIVER Channel (CH:81438D00) for Virtual Circuit (VC:81048A80) HOWS0F --
$!State: 0004 open                Status: 6B path,open,rhwaval,tight,fast
$!                                ECS Status: Tight,Fast
$!BUS: 811A2600 (EWA)  Lcl Device: EW_DE500  Lcl LAN Address: 00-10-64-30-F1-B2
$!Rmt BUS Name:  EWA   Rmt Device: EW_DE500  Rmt LAN Address: 00-10-64-31-16-0C
$ loop4:
$	read/end=eof4 tmp rec
$	crec = f$edit(rec,"TRIM,COMPRESS")
$	if f$extract(0,19,crec) .eqs. "-- PEDRIVER Channel"
$	then
$		rmt_node = f$element(8," ",crec)	!Remote node's name
$		goto loop4
$	endif
$	if f$extract(0,4,rec) .eqs. "BUS:"
$	then
$		loc_dev_name = f$element(2," ",crec) - "(" - ")"	!e.g. EWA
$		loc_dev_type = f$element(5," ",crec)		!e.g. EW_DEGPA
$!!!		loc_dev_addr = f$elmenet(9," ",crec)	!MAC address
$		goto loop4
$	endif
$	if f$extract(0,9,rec) .eqs. "Rmt Name:" .or. f$extract(0,13,rec) .eqs. -
			"Rmt BUS Name:"
$	then
$		crec = crec - "Rmt Name: " - "Rmt BUS Name: "
$		rmt_dev_name = f$element(0," ",crec)
$		rmt_dev_type = f$element(3," ",crec)
$!!!		rmt_dev_addr = f$element(7," ",crec)
$! Don't display the local loopback path
$		if loc_dev_name .nes. "LCL"
$		then
$		    write sys$output -
			f$fao("[!AS]: !AS <!AS> ----- <!AS> !AS :[!AS]",-
			this_node,loc_dev_name,loc_dev_type,-
			rmt_dev_type,rmt_dev_name,rmt_node)
$		    data_found = "T"
$		endif
$!		goto loop4
$	endif
$	goto loop4
$ eof4:
$	close tmp
$	delete 'temp_file'_list3;*	!!!
$ sca_cleanup:
$! If we turned on elevated privileges, turn them back off again
$	if ELEVATED_PRIVS then set process/privilege=('remove_privs')
$	if data_found
$	then
$		write sys$output -
	    "Channel data collection successful on node ",this_node
$	else
$		write sys$output -
	    "*** Error: Data collection failed on node ",this_node," ***"
$	endif
$	exit
$!
$!-----------------------------------------------------------------------------
$!
$ SHOWLAN:
$!
$! SHOWLAN.COM
$!
$! Modified to accommondate DECnet Phase IV style [physical] addresses.
$! (DECnet Phase V uses only the hardware addresses)
$!
$! Check for the privileges required to run SDA
$	auth_privs = "," + f$getjpi("","AUTHPRIV") + ","
$	cur_privs  = "," + f$getjpi("","CURPRIV")  + ","
$! Remember whether or not we had to elevate privileges
$	ELEVATED_PRIVS = "F"	!Assume process already had CMKRNL privilege turned on
$	remove_privs = ""
$! Check for CMKRNL privilege and turn it on if need be
$	if f$locate(",CMKRNL,",cur_privs) .eq. f$length(cur_privs)
$	then	!CMKRNL privilege is not presently turned on
$		if f$locate(",CMKRNL,",auth_privs) .ne. f$length(auth_privs) .or. -
		   f$locate(",SETPRV,",auth_privs) .ne. f$length(auth_privs)
$		then
$			set process/privilege=(CMKRNL)
$			remove_privs = remove_privs + ",noCMKRNL"
$			ELEVATED_PRIVS = "T"
$		else
$			write sys$output -
		    "This procedure requires CMKRNL privilege to run SDA."
$			exit
$		endif
$	endif
$! Check for READALL privilege and turn it on if possible
$	if f$locate(",READALL,",cur_privs) .eq. f$length(cur_privs)
$	then	!READALL privilege is not presently turned on
$		if f$locate(",READALL,",auth_privs) .ne. f$length(auth_privs) .or. -
		   f$locate(",SETPRV,",auth_privs) .ne. f$length(auth_privs)
$		then
$			set process/privilege=(READALL)
$			remove_privs = remove_privs + ",noREADALL"
$			ELEVATED_PRIVS = "T"
$!		else
$! Most systems seem to run SDA with only CMKRNL -- I've only hit a few
$! that apparently have protection problems or something and require READALL.
$! If the user doesn't have READALL, we'll just try SDA anyway without it.
$!			write sys$output -
$!_!		    "This procedure requires READALL privilege to run SDA."
$!			exit
$		endif
$	endif
$	remove_privs = remove_privs - ","	!Remove leading comma
$!
$	write sys$output "Node ",f$getsyi("NODENAME")
$	pid = f$getjpi("","PID")
$	temp_file := showlan_temp_'pid'.temp_file
$	call show_lan/out='temp_file'
$	search 'temp_file' "(SCA)","Hardware Address","Physical Address" -	
		/out='temp_file-1
$!	if $severity .eq. 3 then -	!No matches
$!_!		write sys$output -
$!_!    "*** Error: No MAC address data found on node ",f$getsyi("NODENAME")," ***"
$	delete 'temp_file';*
$	search/window=(0,2) 'temp_file-1 "(SCA)"
$	if $severity .eq. 3 then -	!No matches
		write sys$output -
    "*** Error: No MAC address data found on node ",f$getsyi("NODENAME")," ***"
$	delete 'temp_file-1;*
$! If we turned on elevated privileges, turn them back off again
$	if ELEVATED_PRIVS then set process/privilege=('remove_privs')
$	exit
$!
$ show_lan: subroutine
$	 analyze/system
show lan/full
exit
$	endsubroutine
$!
$!-----------------------------------------------------------------------------
$!
$ BUILD:
$!
$! LAVC_BUILD.COM
$!
$! Assemble/Compile and link LAVC$FAILURE_ANALYSIS.MAR in the defalt directory
$!
$	proc = f$environment("procedure")
$	proc_dir = f$parse(proc,,,"DEVICE")+ f$parse(proc,,,"DIRECTORY")
$	arch = f$getsyi("ARCH_NAME")
$	if arch .eqs. "VAX"
$	then
$		macro_qual = ""
$	else	!Alpha (or presumably IA-64?)
$		macro_qual = "/migrate"
$	endif
$	macro'macro_qual'/list=lavc$failure_analysis/obj=lavc$failure_analysis -
		sys$library:arch_defs.mar+-
		'proc_dir'lavc$failure_analysis.mar+-
		sys$library:lib.mlb/lib
$	link/map/exe lavc$failure_analysis
$	exit
