$ if_standalone = ""	!"" for use as a standalone procedure; "!" for use
$!			!within SHOW_PATHS.COM or SHOW_SEGMENTS.COM.
$!
$! SHOW_ALL_CHANNELS.COM
$!
$! Show all the active LAN channels (paths) seen by PEDRIVER to each remote node
$!
$! Based on SHOW_PREF_PATH.COM, SHOW_ALL_PATHS.COM
$!						V1.0 3/2001
$! Author: Keith Parris  parris@encompasserve.org or keithparris@yahoo.com
$! http://www.geocities.com/keithparris/ or http://encompasserve.org/~parris/
$! Modified for 7.3-1    KP 10/8/2002
$!	Output format changed for SDA> SHOW PORTS/VC/NODE=xxx
$! Improved		 KP 12/4/2002
$!	VAX SDA apparently doesn't support /NOINDEX qualifer to SET OUTPUT command
$!	Modified to also show paths between different LAN adapters on the same
$!	node, to fix problems when used for SHOW_SEGMENTS.COM, with its need to
$!	map adapters to network segments.
$!	Improved error-checking and privilege handling
$!
$	if_debug = "!"	!Use "!" for normal operations, and "" for debugging
$! Check to see if SCS communications over the LAN is even enabled on this node.
$! If not, SDA> SHOW PORT will not define the PE_PDT symbol and we can't
$! gather any data.
$	if (.not. f$getdvi("PEA0","EXISTS")) .or. -
	   (.not. f$getsyi("NISCS_LOAD_PEA0"))
$	then
$	    write sys$output "LAN SCS communications is disabled on node ",-
			f$getsyi("NODENAME"),"."
$!	    write sys$output "NISCS_LOAD_PEA0 parameter value is ",-
$!_!			f$getsyi("NISCS_LOAD_PEA0"),"."
$		exit
$	endif
$!
$	data_found = "F"	!Assume we failed until proven otherwise
$! Check for the privileges required to run SDA
$	auth_privs = f$getjpi("","AUTHPRIV")
$	cur_privs = f$getjpi("","CURPRIV")
$! Remember whether or not we had to elevate privileges
$	ELEVATED_PRIVS = "F"	!Assume process already had CMKRNL privilege turned on
$	remove_privs = ""
$! Check for CMKRNL privilege and turn it on if need be
$	if f$locate("CMKRNL,",cur_privs) .eq. f$length(cur_privs)
$	then	!CMKRNL privilege is not presently turned on
$		if f$locate("CMKRNL,",auth_privs) .ne. f$length(auth_privs) .or. -
		   f$locate("SETPRV,",auth_privs) .ne. f$length(auth_privs)
$		then
$			set process/privilege=(CMKRNL)
$			remove_privs = remove_privs + ",noCMKRNL"
$			ELEVATED_PRIVS = "T"
$		else
$			write sys$output -
		    "This procedure requires CMKRNL privilege to run SDA."
$			exit
$		endif
$	endif
$! Check for READALL privilege and turn it on if possible
$	if f$locate("READALL,",cur_privs) .eq. f$length(cur_privs)
$	then	!READALL privilege is not presently turned on
$		if f$locate("READALL,",auth_privs) .ne. f$length(auth_privs) .or. -
		   f$locate("SETPRV,",auth_privs) .ne. f$length(auth_privs)
$		then
$			set process/privilege=(READALL)
$			remove_privs = remove_privs + ",noREADALL"
$			ELEVATED_PRIVS = "T"
$!		else
$! Most systems seem to run SDA with only CMKRNL -- I've only hit a few
$! that apparently have protection problems or something and require READALL.
$! If the user doesn't have READALL, we'll just try SDA anyway without it.
$!			write sys$output -
$!_!		    "This procedure requires READALL privilege to run SDA."
$!			exit
$		endif
$	endif
$	remove_privs = remove_privs - ","	!Remove leading comma
$! Make temporary filenames unique by PID
$!In captive mode, use temporary filenames from main procedure
$ 'if_standalone' pid = f$getjpi("","PID")
$ 'if_standalone' temp_file := show_all_channels_temp_'pid'.temp_file
$	delete := delete
$! First, find out the names of all the nodes in the cluster
$	this_node = f$getsyi("NODENAME")	!This is where we're running
$ 'if_debug' show symbol this_node
$	nodecount = 0
$	context = ""
$ loop:
$	csid = f$csid(context)	!Get the CSID of the next cluster member
$	if csid .nes. ""
$	then
$	    nodecount = nodecount + 1	!1 more node
$	    nodename_'nodecount' = f$getsyi("NODENAME",,csid)
$ 'if_debug' show symbol nodename_'nodecount'
$	    goto loop
$	endif
$ 'if_debug' show symbol nodecount
$! Next, use SDA to find out the channel addresses of all the LAN paths
$! to each node
$	close/nolog com
$	open/write com 'temp_file'_com
$	write com "$ DEFINE/USER SYS$OUTPUT NLA0:"
$	write com "$ DEFINE/USER SYS$ERROR NLA0:"
$	write com "$ ANALYZE/SYSTEM"
$	out_qual = "/NOINDEX"
$	arch = f$getsyi("ARCH_NAME")
$	if arch .eqs. "VAX" then out_qual = ""
$	write com "SET OUTPUT''out_qual' _NLA0:"
$	write com "SHOW PORTS"	!To set symbols for use later
$	write com "SET OUTPUT''out_qual' ",temp_file,"_LIST"
$	i = 0
$ loop2:
$	i = i + 1
$!(Don't bother to look up a path to this node itself)
$!! Actually, we DO need to look up paths to this node itself.  We'll ignore
$!! the loopback path and path from an adapter to itself, but we will need to
$!! know (for SHOW_PATHS.COM and SHOW_SEGMENTS.COM) whenever there is a path
$!! between two different LAN adapters on the same node, because that indicates
$!! that they are connected to the same VLAN or extended LAN or network segment,
$!! however you want to phrase it.
$!!	if nodename_'i' .nes. this_node then-
$		write com "SHOW PORT/ADDRESS=PE_PDT/VC/NODE=",nodename_'i'
$	if i .lt. nodecount then goto loop2
$	write com "EXIT"
$	write com "$ EXIT"
$	close com
$ 'if_debug' type/page/header 'temp_file'_com
$	@'temp_file'_com
$! copy 'temp_file'_com chan.com
$	delete 'temp_file'_com;*	!!!
$	if f$search("''temp_file'_list") .eqs. ""
$	then	!SDA must have failed to write the file
$		write sys$output -
    "*** Error: SDA> SHOW PORT output failure on node ",this_node," ***"
$		goto cleanup
$	endif
$ 'if_debug' type/page/header 'temp_file'_list
$!Suppress "%SEARCH-I-NOMATCHES, no strings matched
$! and      %SEARCH-I-NULLFILE, file xxxxxx contains no records
$	define/user sys$output _NLA0:
$	define/user sys$error _NLA0:
$	search 'temp_file'_list "Preferred","Active","path,open"-
		/out='temp_file'_channels
$	search_severity = $severity
$ 'if_debug' show symbol search_severity
$	delete 'temp_file'_list;*	!!!
$	if search_severity .eq. 3	!No channel data found
$	then	!SDA must have failed to write anything useful to the file
$		write sys$output -
    "*** Error: SDA> SHOW PORT output empty on node ",this_node," ***"
$		goto cleanup
$	endif
$ 'if_debug' type/page/header 'temp_file'_channels
$	close/nolog chan
$! Now use SDA again to find out the "bus" (LAN adapter) which is currently
$! used in the path to each node
$	open/write com 'temp_file'_com2
$	write com "$ DEFINE/USER SYS$OUTPUT _NLA0:"
$	write com "$ DEFINE/USER SYS$ERROR _NLA0:"
$	write com "$ ANALYZE/SYSTEM"
$	write com "SET OUTPUT''out_qual' ",temp_file,"_LIST2"
$	open/read chan 'temp_file'_channels
$!
$! Pick up addresses of data structures for all channels that are working.
$! Here's what we're parsing:
$!
$!Address     Type    Xmt Time Size Preferred    Best        Last State Change
$!--------  --------- -------- ---- ---------  --------   -----------------------
$!81764F00  Preferred 00C59486 1412       289         4   28-FEB-2001 15:25:16.19
$!81764E00  Active    00C59486 1412         3       201   28-FEB-2001 15:25:15.83
$!
$! For 7.3-1:
$!Address   LCL/RMT   State      ECS   Size  AvgRTT(us)  Last State Change
$!--------  -------  --------   -----  ----  ----------  -----------------------
$!812D8540  EWA/EWA  path,open  T/P/S  1426     24220.9   7-OCT-2002 21:36:57.12
$!812B9340  EWB/EWB  path,open  T/P/F  1426       244.1   7-OCT-2002 21:24:40.58
$ loop3:
$	read/end=eof3 chan rec
$	rec = f$edit(rec,"COMPRESS,TRIM")
$!Try the 7.3-1 variant first
$	state = f$element(2," ",rec)
$	if state .eqs. "path,open" then goto got_chan
$	path_type = f$element(1," ",rec)
$	if path_type .nes. "Preferred" .and. path_type .nes. "Active" then -
		goto loop3
$ got_chan:
$!! show symbol rec
$	chan_addr = f$element(0," ",rec)
$!! We're looking for an 8-digit hexadecimal address
$!!	if f$length(chan_addr) .ne. 8 then goto loop3
$	write com "SHOW PORT/CHANNEL=",chan_addr," !"	!,path_type
$	goto loop3
$ eof3:
$	close chan
$	delete 'temp_file'_channels;*	!!!
$	close com
$ 'if_debug' type/page/header 'temp_file'_com2
$	@'temp_file'_com2
$	delete 'temp_file'_com2;*	!!!
$	if f$search("''temp_file'_list2") .eqs. ""
$	then	!SDA must have failed to write the file
$		write sys$output -
    "*** Error: SDA> SHOW CHANNEL output failure on node ",this_node," ***"
$		goto cleanup
$	endif
$ 'if_debug' type/page/header 'temp_file'_list2
$!
$! Show the user what we found
$!
$!Suppress "%SEARCH-I-NOMATCHES, no strings matched
$! and      %SEARCH-I-NULLFILE, file xxxxxx contains no records
$	define/user sys$output _NLA0:
$	define/user sys$error _NLA0:
$	search 'temp_file'_list2 -
		"PEDRIVER Channel","Lcl Device:","Rmt Device:"-
		/out='temp_file'_list3
$	search_severity = $severity
$ 'if_debug' show symbol search_severity
$	delete 'temp_file'_list2;*	!!!
$	if search_severity .eq. 3
$	then	!SDA must have failed to write anything useful to the file
$		write sys$output -
    "*** Error: SDA> SHOW CHANNEL output empty on node ",this_node," ***"
$		goto cleanup
$	endif
$ 'if_debug' type/page/header 'temp_file'_list3
$	close/nolog tmp
$	open/read tmp  'temp_file'_list3
$! Here's an example of what we're parsing in loop 4:
$! -- PEDRIVER Channel (CH:81A71D40) for Virtual Circuit (VC:81A5FA40) NCACAB --
$!BUS: 819CB580  (EWB)  Lcl Device: EW_DEGPA  Lcl LAN Address: 00-60-6D-21-32-B4
$!Rmt Name: EWB         Rmt Device: EW_DEGPA  Rmt LAN Address: 00-60-6D-21-32-C0
$! For 7.3-1:
$! -- PEDRIVER Channel (CH:81438D00) for Virtual Circuit (VC:81048A80) HOWS0F --
$!State: 0004 open                Status: 6B path,open,rhwaval,tight,fast
$!                                ECS Status: Tight,Fast
$!BUS: 811A2600 (EWA)  Lcl Device: EW_DE500  Lcl LAN Address: 00-10-64-30-F1-B2
$!Rmt BUS Name:  EWA   Rmt Device: EW_DE500  Rmt LAN Address: 00-10-64-31-16-0C
$ loop4:
$	read/end=eof4 tmp rec
$	crec = f$edit(rec,"TRIM,COMPRESS")
$	if f$extract(0,19,crec) .eqs. "-- PEDRIVER Channel"
$	then
$		rmt_node = f$element(8," ",crec)	!Remote node's name
$		goto loop4
$	endif
$	if f$extract(0,4,rec) .eqs. "BUS:"
$	then
$		loc_dev_name = f$element(2," ",crec) - "(" - ")"	!e.g. EWA
$		loc_dev_type = f$element(5," ",crec)		!e.g. EW_DEGPA
$!!!		loc_dev_addr = f$elmenet(9," ",crec)	!MAC address
$		goto loop4
$	endif
$	if f$extract(0,9,rec) .eqs. "Rmt Name:" .or. f$extract(0,13,rec) .eqs. -
			"Rmt BUS Name:"
$	then
$		crec = crec - "Rmt Name: " - "Rmt BUS Name: "
$		rmt_dev_name = f$element(0," ",crec)
$		rmt_dev_type = f$element(3," ",crec)
$!!!		rmt_dev_addr = f$element(7," ",crec)
$! Don't display the local loopback path
$		if loc_dev_name .nes. "LCL"
$		then
		    write sys$output -
			f$fao("[!AS]: !AS <!AS> ----- <!AS> !AS :[!AS]",-
			this_node,loc_dev_name,loc_dev_type,-
			rmt_dev_type,rmt_dev_name,rmt_node)
$		    data_found = "T"
$		endif
$!		goto loop4
$	endif
$	goto loop4
$ eof4:
$	close tmp
$	delete 'temp_file'_list3;*	!!!
$ cleanup:
$! If we turned on elevated privileges, turn them back off again
$	if ELEVATED_PRIVS then set process/privilege=('remove_privs')
$	if data_found
$	then
$		write sys$output -
	    "Channel data collection successful on node ",this_node
$	else
$		write sys$output -
	    "*** Error: Data collection failed on node ",this_node," ***"
$	endif
$	exit
